////////////////////////////////////////////////////////////////////
// CVI_run6.c
// modified : 12 January 2000
// purpose: run C-842 using QFLW400.DLL under LabWindows CVI
// -----------------------------------------------------------------
//
// Note : (01 OCT 1998)
// due to an internal converting problem inside the QFLW32F.DLL, the report
// of the execute function is cut after the third character. Use the translate
// function instead.
// Despite this malfunction, execute can be used sending commands as it olso 
// can be done with the translate function.
//
// This peport problem will be fixed with the next release of the QFLW-DLL.
// -----------------------------------------------------------------  
#include <ansi_c.h>
#include "QFL400_CVI.h"

C842init()
{
  char report[128];
  BYTE nAxis;
  
  printf("starting initialization :\n");
  if (Board_Installed())
       printf("board found\n");
  else
     {
       printf("board not found\n");
       return 0;
     }
  
  nAxis = Axis_Installed();
  printf("Number of axes found: %i\n",nAxis);
  
  InitBoard(9);
  InitAxis(1);
  InitAxis(2);

  autodetect();  // do not move this command
  return 0;
}


/////////////////////////////////////////////////////////////////
WaitMoveComplete()
{
  char report[128];

  while (moving(1))
    // loop until the move is done or the limit switch is hit
    {
       translate("0TP",report);
       printf("%s\n",report);
    }
  while (moving(2))
    // loop until the move is done or the limit switch is hit
    {
       translate("0TP",report);
       printf("%s\n",report);
    }
  
    printf(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>move complete\n");
    
    return 0;
}  

/////////////////////////////////////////////////////////////////
TestVectorFunctions()
{
  char report[128];

  VectorA(1,2,45000,200000,50000); 
  WaitMoveComplete();

  VectorR(1,2,45000,200000,50000); 
  WaitMoveComplete();
  
  MoveR_12(40000,-100000);         // absolutepos_X, absolutepos_Y
  WaitMoveComplete();

  MoveA_12(40000,150000);          // absolutepos_X, absolutepos_Y
  WaitMoveComplete();
 
  return 0;
}

/////////////////////////////////////////////////////////////////
TestLimitSwitches()
{
  char report[128];

  translate("1SV20000",report);
  translate("1MR60000",report);

  
  while (moving(1))

    // loop until the move is done or the limit switch is hit
    {
       translate("0TP",report);
       printf("%s\n",report);
    }


  translate("1MR-60000",report);
  printf(" now moving in negative direction \n");

  while (moving(1))
    {
       translate("0TP",report);
       printf("%s\n",report);
    }

  return 0;
}

/////////////////////////////////////////////////////////////////
//
// TestLimitSwitches_2
//
// example how to withdraw the stage from the limit switch.
// Note that the withdraw_length has to be long enought to 
// release the limit switch.
//
/////////////////////////////////////////////////////////////////

TestLimitSwitches_2()
{
  char  report[128];
  long  position_when_hit, target_new;
  const withdraw_length = 30000;

  printf(">>>>>>>>>>>>>>>>>>>>>>>>>>  now starting a long move\n");

  translate("1MR500000",report);
  while (moving(1))
    // loop until the move is done or the limit switch is hit
    {
       translate("0TP",report);   // read positions of all 4 axes
       printf("%s\n",report);     // show it
    }

  // translate("1MR-80000",report); This would cause the motor to move to 
  // absolute 500000-80000=420000, causing the motor to start deeper into 
  // the limit switch.
  
  // use this instead :

  position_when_hit = get_pos(1);
  target_new = position_when_hit - withdraw_length;
  execute(1,MA,target_new,report);
  
  printf(">>>>>>>>>>>>>>>>>>>>>>>>>>  now withdrawing %i steps\n",withdraw_length);
  
  while (moving(1))
    // loop until the move is done or the limit switch is hit again
    {
       translate("0TP",report);   // read positions of all 4 axes
       printf("%s\n",report);     // show it
    }



   return 0;
}   

//////////////////////////////////////////////////////////////////
main()
//////////////////////////////////////////////////////////////////
{
  char report[128];
  int i;
  
  // After power-up, the board has to be initialized by the C842init function
  // Consecutive program runs do not require re-initialization.

     C842init();


  translate("1SV12000",report);

  TestLimitSwitches_2();
  TestLimitSwitches_2();
  TestLimitSwitches_2();
  TestLimitSwitches_2();
  TestLimitSwitches_2();
  TestLimitSwitches_2();

  return 0;  // stop program

  TestVectorFunctions();
  
  return 0;  // stop program

  WaitStop(1);
  
  translate("1SV20000",report);
  printf("-----------\n");

  MoveR(1, 10000);			 // 1st option for relative move
  printf("waiting...\n");
  
  WaitStop(1);
  translate("0TP",report);
  printf("%s\n",report);
  
  
  translate("1MR10000",report);   // 2nd option for relative move
  printf("waiting...\n");
  WaitStop(1);
  translate("0TP",report);
  printf("%s\n",report);
  
  execute(1,MR,10000,report);	 // 3rd option for relative move
  printf("waiting...\n");
  WaitStop(1);
  translate("0TP",report);
  printf("%s\n",report);

  
  execute(1,MR,40000,report);
  for(i=0;i<7;i++)
    { 
      translate("0TP",report);
      printf("%s\n",report);

    }
  
  execute(1,TP,0,report);
  //printf(report);
  printf("%s\n",report);

  
  //execute(1,MR,-50000,report);
  //printf("%s\n",report);
  
  translate("1MR-30000",report);
  printf("%s\n",report);
  
  //execute(1,TP,0,report);
  translate("1TP",report);
  printf("%s\n",report);

}

