#include <windows.h>
#include <QFLW500_CVI.h>

/* The two macros below are used as error return codes */
/* in case the DLL does not load, or is missing one or */
/* more functions, respectively.  You must define them */
/* to whatever values are meaningful for your DLL.     */
#define kFailedToLoadDLLError     ???
#define kCouldNotFindFunction     ???

static HINSTANCE DLLHandle;

/* Declare the variables that hold the addresses of the function   */
/* pointers.                                                       */
static long (__stdcall *get_pos_Ptr)(BYTE axis);
static long (__stdcall *get_PosErr_Ptr)(BYTE axis);
static long (__stdcall *get_v_Ptr)(BYTE axis);
static int (__stdcall *get_stat_Ptr)(BYTE axis);
static void (__stdcall *get_pos4_Ptr)(TQ_long *pos4);
static WORD (__stdcall *get_BoardStatus_Ptr)(void);
static BYTE (__stdcall *get_DIN_Ptr)(void);
static int (__stdcall *get_ErrorStatus_Ptr)(void);
static int (__stdcall *get_Limit_Ptr)(void);
static int (__stdcall *get_LimitAxis_Ptr)(BYTE axis);
static WORD (__stdcall *get_RefSignal_Ptr)(BYTE axis);
static void (__stdcall *execute_Ptr)(BYTE axis, TCMD command, long para,
                                     char *report);
static void (__stdcall *translate_Ptr)(char *comstr, char *report);
static BOOL (__stdcall *moving_Ptr)(BYTE axis);
static void (__stdcall *Set_BaseAddress_Ptr)(unsigned short address);
static BYTE (__stdcall *Axis_Installed_Ptr)(void);
static int (__stdcall *InitBoard_Ptr)(BYTE mode);
static void (__stdcall *InitAxis_Ptr)(BYTE axis);
static void (__stdcall *InitStage_Ptr)(BYTE axis, BYTE stagetype);
static BOOL (__stdcall *Board_Installed_Ptr)(void);
static void (__stdcall *MoveA_Ptr)(BYTE axis, long nPosition);
static void (__stdcall *MoveR_Ptr)(BYTE axis, long nCounts);
static void (__stdcall *VectorA_Ptr)(BYTE axis1, BYTE axis2, long p1,
                                     long p2, long vv, long va);
static void (__stdcall *VectorR_Ptr)(BYTE axis1, BYTE axis2, long inc1,
                                     long inc2, long vv, long va);
static int (__stdcall *WaitStop_Ptr)(BYTE axis);
static void (__stdcall *Init_LS_Ptr)(BYTE axis1, BYTE axis2, BYTE axis3,
                                     BYTE axis4);
static void (__stdcall *axis_select_Ptr)(BYTE axis);
static void (__stdcall *setQMC_Ptr)(BYTE command, long parameter);
static long (__stdcall *getQMC_Ptr)(BYTE command);
static int (__stdcall *autodetect_Ptr)(void);
static void (__stdcall *set_pos_Ptr)(BYTE axis, long newpos);
static int (__stdcall *set_Amplifier_Ptr)(BYTE mode);
static void (__stdcall *set_DOUT_Ptr)(BYTE digout);
static void (__stdcall *set_acc_Ptr)(BYTE axis, int acceleration);
static void (__stdcall *set_pid_Ptr)(BYTE axis, int pterm, int iterm,
                                     int dterm);
static void (__stdcall *set_vel_Ptr)(BYTE axis, int velocity);
static void (__stdcall *AutoFindEdge_Ptr)(BYTE axis, BYTE mode);
static void (__stdcall *MoveA_12_Ptr)(long pos_1, long pos_2);
static void (__stdcall *MoveA_123_Ptr)(long pos_1, long pos_2, long pos_3);
static void (__stdcall *MoveR_12_Ptr)(long shift_1, long shift_2);
static void (__stdcall *MoveR_123_Ptr)(long shift_1, long shift_2,
                                       long shift_3);


/* Load the DLL and get the addresses of the functions */
static int LoadDLLIfNeeded(void)
{
    if (DLLHandle)
        return 0;

    DLLHandle = LoadLibrary("QFLW500.dll");
    if (DLLHandle == NULL) {
        return kFailedToLoadDLLError;
        }

    if (!(get_pos_Ptr = (void*) GetProcAddress(DLLHandle, "get_pos")))
        goto FunctionNotFoundError;

    if (!(get_PosErr_Ptr = (void*) GetProcAddress(DLLHandle, "get_PosErr")))
        goto FunctionNotFoundError;

    if (!(get_v_Ptr = (void*) GetProcAddress(DLLHandle, "get_v")))
        goto FunctionNotFoundError;

    if (!(get_stat_Ptr = (void*) GetProcAddress(DLLHandle, "get_stat")))
        goto FunctionNotFoundError;

    if (!(get_pos4_Ptr = (void*) GetProcAddress(DLLHandle, "get_pos4")))
        goto FunctionNotFoundError;

    if (!(get_BoardStatus_Ptr = (void*) GetProcAddress(DLLHandle, 
         "get_BoardStatus")))
        goto FunctionNotFoundError;

    if (!(get_DIN_Ptr = (void*) GetProcAddress(DLLHandle, "get_DIN")))
        goto FunctionNotFoundError;

    if (!(get_ErrorStatus_Ptr = (void*) GetProcAddress(DLLHandle, 
         "get_ErrorStatus")))
        goto FunctionNotFoundError;

    if (!(get_Limit_Ptr = (void*) GetProcAddress(DLLHandle, "get_Limit")))
        goto FunctionNotFoundError;

    if (!(get_LimitAxis_Ptr = (void*) GetProcAddress(DLLHandle, "get_LimitAxis")))
        goto FunctionNotFoundError;

    if (!(get_RefSignal_Ptr = (void*) GetProcAddress(DLLHandle, "get_RefSignal")))
        goto FunctionNotFoundError;

    if (!(execute_Ptr = (void*) GetProcAddress(DLLHandle, "execute")))
        goto FunctionNotFoundError;

    if (!(translate_Ptr = (void*) GetProcAddress(DLLHandle, "translate")))
        goto FunctionNotFoundError;

    if (!(moving_Ptr = (void*) GetProcAddress(DLLHandle, "moving")))
        goto FunctionNotFoundError;

    if (!(Set_BaseAddress_Ptr = (void*) GetProcAddress(DLLHandle, 
         "Set_BaseAddress")))
        goto FunctionNotFoundError;

    if (!(Axis_Installed_Ptr = (void*) GetProcAddress(DLLHandle, 
         "Axis_Installed")))
        goto FunctionNotFoundError;

    if (!(InitBoard_Ptr = (void*) GetProcAddress(DLLHandle, "InitBoard")))
        goto FunctionNotFoundError;

    if (!(InitAxis_Ptr = (void*) GetProcAddress(DLLHandle, "InitAxis")))
        goto FunctionNotFoundError;

    if (!(InitStage_Ptr = (void*) GetProcAddress(DLLHandle, "InitStage")))
        goto FunctionNotFoundError;

    if (!(Board_Installed_Ptr = (void*) GetProcAddress(DLLHandle, 
         "Board_Installed")))
        goto FunctionNotFoundError;

    if (!(MoveA_Ptr = (void*) GetProcAddress(DLLHandle, "MoveA")))
        goto FunctionNotFoundError;

    if (!(MoveR_Ptr = (void*) GetProcAddress(DLLHandle, "MoveR")))
        goto FunctionNotFoundError;

    if (!(VectorA_Ptr = (void*) GetProcAddress(DLLHandle, "VectorA")))
        goto FunctionNotFoundError;

    if (!(VectorR_Ptr = (void*) GetProcAddress(DLLHandle, "VectorR")))
        goto FunctionNotFoundError;

    if (!(WaitStop_Ptr = (void*) GetProcAddress(DLLHandle, "WaitStop")))
        goto FunctionNotFoundError;

    if (!(Init_LS_Ptr = (void*) GetProcAddress(DLLHandle, "Init_LS")))
        goto FunctionNotFoundError;

    if (!(axis_select_Ptr = (void*) GetProcAddress(DLLHandle, "axis_select")))
        goto FunctionNotFoundError;

    if (!(setQMC_Ptr = (void*) GetProcAddress(DLLHandle, "setQMC")))
        goto FunctionNotFoundError;

    if (!(getQMC_Ptr = (void*) GetProcAddress(DLLHandle, "getQMC")))
        goto FunctionNotFoundError;

    if (!(autodetect_Ptr = (void*) GetProcAddress(DLLHandle, "autodetect")))
        goto FunctionNotFoundError;

    if (!(set_pos_Ptr = (void*) GetProcAddress(DLLHandle, "set_pos")))
        goto FunctionNotFoundError;

    if (!(set_Amplifier_Ptr = (void*) GetProcAddress(DLLHandle, "set_Amplifier")))
        goto FunctionNotFoundError;

    if (!(set_DOUT_Ptr = (void*) GetProcAddress(DLLHandle, "set_DOUT")))
        goto FunctionNotFoundError;

    if (!(set_acc_Ptr = (void*) GetProcAddress(DLLHandle, "set_acc")))
        goto FunctionNotFoundError;

    if (!(set_pid_Ptr = (void*) GetProcAddress(DLLHandle, "set_pid")))
        goto FunctionNotFoundError;

    if (!(set_vel_Ptr = (void*) GetProcAddress(DLLHandle, "set_vel")))
        goto FunctionNotFoundError;

    if (!(AutoFindEdge_Ptr = (void*) GetProcAddress(DLLHandle, "AutoFindEdge")))
        goto FunctionNotFoundError;

    if (!(MoveA_12_Ptr = (void*) GetProcAddress(DLLHandle, "MoveA_12")))
        goto FunctionNotFoundError;

    if (!(MoveA_123_Ptr = (void*) GetProcAddress(DLLHandle, "MoveA_123")))
        goto FunctionNotFoundError;

    if (!(MoveR_12_Ptr = (void*) GetProcAddress(DLLHandle, "MoveR_12")))
        goto FunctionNotFoundError;

    if (!(MoveR_123_Ptr = (void*) GetProcAddress(DLLHandle, "MoveR_123")))
        goto FunctionNotFoundError;

    return 0;

FunctionNotFoundError:
    FreeLibrary(DLLHandle);
    DLLHandle = 0;
    return kCouldNotFindFunction;
}


/* Glue Code for each of the DLL functions */



long __stdcall get_pos(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_pos_Ptr)(axis);
}


long __stdcall get_PosErr(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_PosErr_Ptr)(axis);
}


long __stdcall get_v(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_v_Ptr)(axis);
}


int __stdcall get_stat(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_stat_Ptr)(axis);
}


void __stdcall get_pos4(TQ_long *pos4)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*get_pos4_Ptr)(pos4);
}


WORD __stdcall get_BoardStatus(void)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_BoardStatus_Ptr)();
}


BYTE __stdcall get_DIN(void)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_DIN_Ptr)();
}


int __stdcall get_ErrorStatus(void)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_ErrorStatus_Ptr)();
}


int __stdcall get_Limit(void)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_Limit_Ptr)();
}


int __stdcall get_LimitAxis(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_LimitAxis_Ptr)(axis);
}


WORD __stdcall get_RefSignal(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*get_RefSignal_Ptr)(axis);
}


void __stdcall execute(BYTE axis, TCMD command, long para, char *report)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*execute_Ptr)(axis, command, para, report);
}


void __stdcall translate(char *comstr, char *report)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*translate_Ptr)(comstr, report);
}


BOOL __stdcall moving(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*moving_Ptr)(axis);
}


void __stdcall Set_BaseAddress(unsigned short address)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*Set_BaseAddress_Ptr)(address);
}


BYTE __stdcall Axis_Installed(void)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*Axis_Installed_Ptr)();
}


int __stdcall InitBoard(BYTE mode)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*InitBoard_Ptr)(mode);
}


void __stdcall InitAxis(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*InitAxis_Ptr)(axis);
}


void __stdcall InitStage(BYTE axis, BYTE stagetype)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*InitStage_Ptr)(axis, stagetype);
}


BOOL __stdcall Board_Installed(void)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*Board_Installed_Ptr)();
}


void __stdcall MoveA(BYTE axis, long nPosition)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*MoveA_Ptr)(axis, nPosition);
}


void __stdcall MoveR(BYTE axis, long nCounts)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*MoveR_Ptr)(axis, nCounts);
}


void __stdcall VectorA(BYTE axis1, BYTE axis2, long p1, long p2, long vv, 
                       long va)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*VectorA_Ptr)(axis1, axis2, p1, p2, vv, va);
}


void __stdcall VectorR(BYTE axis1, BYTE axis2, long inc1, long inc2, 
                       long vv, long va)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*VectorR_Ptr)(axis1, axis2, inc1, inc2, vv, va);
}


int __stdcall WaitStop(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*WaitStop_Ptr)(axis);
}


void __stdcall Init_LS(BYTE axis1, BYTE axis2, BYTE axis3, BYTE axis4)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*Init_LS_Ptr)(axis1, axis2, axis3, axis4);
}


void __stdcall axis_select(BYTE axis)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*axis_select_Ptr)(axis);
}


void __stdcall setQMC(BYTE command, long parameter)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*setQMC_Ptr)(command, parameter);
}


long __stdcall getQMC(BYTE command)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*getQMC_Ptr)(command);
}


int __stdcall autodetect(void)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*autodetect_Ptr)();
}


void __stdcall set_pos(BYTE axis, long newpos)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*set_pos_Ptr)(axis, newpos);
}


int __stdcall set_Amplifier(BYTE mode)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return dllLoadError;
    return (*set_Amplifier_Ptr)(mode);
}


void __stdcall set_DOUT(BYTE digout)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*set_DOUT_Ptr)(digout);
}


void __stdcall set_acc(BYTE axis, int acceleration)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*set_acc_Ptr)(axis, acceleration);
}


void __stdcall set_pid(BYTE axis, int pterm, int iterm, int dterm)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*set_pid_Ptr)(axis, pterm, iterm, dterm);
}


void __stdcall set_vel(BYTE axis, int velocity)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*set_vel_Ptr)(axis, velocity);
}


void __stdcall AutoFindEdge(BYTE axis, BYTE mode)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*AutoFindEdge_Ptr)(axis, mode);
}


void __stdcall MoveA_12(long pos_1, long pos_2)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*MoveA_12_Ptr)(pos_1, pos_2);
}


void __stdcall MoveA_123(long pos_1, long pos_2, long pos_3)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*MoveA_123_Ptr)(pos_1, pos_2, pos_3);
}


void __stdcall MoveR_12(long shift_1, long shift_2)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*MoveR_12_Ptr)(shift_1, shift_2);
}


void __stdcall MoveR_123(long shift_1, long shift_2, long shift_3)
{
    int dllLoadError;

    if (dllLoadError = LoadDLLIfNeeded())
        return;
    (*MoveR_123_Ptr)(shift_1, shift_2, shift_3);
}

