(****************************************************************************)
(* File name : QFLW_CmdDEF.PAS                                              *)
(* Purpose   : Type declarations for QFLW500 (Borland Pascal/Delphi)        *)
(* Date      : 2002-02-01                                                   *)
(****************************************************************************)
(*                                                                          *)
(*   Note: all velocity and acceleration settings and readings are adjusted *)
(*         to 400 s sample time. Using 2-axis chipsets require to set the  *)
(*         sample time accordingly.                                         *)
(****************************************************************************)

type
   Tstr = string[5];

   Tcmd = record
             code   : byte;
             qclass : byte;
             lab    : Tstr;
          end;

//   TQ_integer = array[1..4] of integer;

const
   ExtLib = 'QFLW500.DLL';
   BaseAddress : integer = $210;    // default address set at card

   LimitMode   : byte = $FF;        // no limit switches connected
   BoardMode   : byte = $FF;
   MaxAxis     : byte = $FF;
   ErrorStatus : byte = $FF;

//   QPos     : TQ_integer = (0,0,0,0);
//   QPos_old : TQ_integer = (0,0,0,0);
//   QPosErr  : TQ_integer = (0,0,0,0);
//   QStatus  : TQ_integer = (0,0,0,0);

   no_parameter = 0;

  // Command qclasses
   cmd_nopara = 1;
   rd_nofrac  = 2;
   rd_U1616   = 4;
   rd_S1616   = 5;
   rd_U0032   = 6;

   wr_nofrac  = 12;
   wr_U1616   = 14;
   wr_S1616   = 15;
   wr_U0032   = 16;
   exdef      = 20;

  //---------------------------------------------------------------------------
  // Definition of commands as variables of type Tcmd
  // code : Processor hex commands
  // qclass: used for formatting parameter
  // lab  : label that is returned in the report string
  //---------------------------------------------------------------------------
  // Commands beyond the command set of the MC1401 have to be emulated
  // by library functions and are coded beginning with $E1..$FF

//0
   UP : Tcmd = (code:$1A; qclass:cmd_nopara; lab:'UP:' ); (* UPDATE         *)
   TP : Tcmd = (code:$37; qclass:rd_nofrac;  lab:'TP:' ); (* GET_ACTL_POS   *)
   GLS: Tcmd = (code:$67; qclass:rd_nofrac;  lab:'GLS:'); (* GET_LMT_SWTCH  *)
   TT : Tcmd = (code:$4A; qclass:rd_nofrac;  lab:'TT:' ); (* GET_POS        *)
   TS : Tcmd = (code:$31; qclass:rd_nofrac;  lab:'TS:' ); (* GET_STATUS     *)
   GIP: Tcmd = (code:$36; qclass:rd_nofrac;  lab:'GIP:'); (* GET_CAPT       *)
   MA : Tcmd = (code:$10; qclass:exdef;      lab:'MA:' ); (* SET_POS        *)
   SV : Tcmd = (code:$11; qclass:wr_U1616;   lab:'SV:'); // SET_VEL
   GMO: Tcmd = (code:$49; qclass:rd_nofrac;  lab:'GMO:'); (* GET_MODE       *)
   MR : Tcmd = (code:$E1; qclass:exdef;      lab:'MR:' ); (* Move Relative  *)
//1
   GH : Tcmd = (code:$E2; qclass:exdef;      lab:'GH:' ); (* Go Home        *)
   TX : Tcmd = (code:$36; qclass:rd_nofrac;  lab:'TX:' ); (* GET_CAPT       *)
   WS : Tcmd = (code:$E3; qclass:exdef;      lab:'WS:' ); (* Wait Stop      *)
   MN : Tcmd = (code:$43; qclass:exdef;      lab:'MN:' ); (* MTR_ON         *)
   MF : Tcmd = (code:$42; qclass:cmd_nopara; lab:'MF:' ); (* MTR_OFF        *)
   TV : Tcmd = (code:$E4; qclass:exdef;      lab:'TV:' ); (* Tell Mot Velocity*)
   CLR: Tcmd = (code:$33; qclass:cmd_nopara; lab:'CLR:'); (* CLR_STATUS     *)
   ST : Tcmd = (code:$E5; qclass:exdef;      lab:'ST:' ); (* Stop w prog velo *)
   SA : Tcmd = (code:$12; qclass:wr_U1616;   lab:'SA:'); // SET_ACC
   DH : Tcmd = (code:$3F; qclass:cmd_nopara; lab:'DH:' ); (* ZERO_POS       *)
//2
   TY : Tcmd = (code:$4B; qclass:rd_U1616;   lab:'TY:' ); (* GET_VEL        *)
   GPP: Tcmd = (code:$1D; qclass:rd_nofrac;  lab:'GPP:'); (* GET_TRGT_POS   *)
   GPV: Tcmd = (code:$1E; qclass:rd_S1616;   lab:'GPV:'); (* GET_TRGT_VEL   *)
   TF : Tcmd = (code:$60; qclass:rd_nofrac;  lab:'TF:' ); (* GET_ACTL_POS_ERR *)
   RT : Tcmd = (code:$39; qclass:cmd_nopara; lab:'RT: '); (* RESET          *)
   GB : Tcmd = (code:$57; qclass:rd_nofrac;  lab:'GB: '); (* GET_BRK_PNT    *)
   GIR: Tcmd = (code:$30; qclass:rd_nofrac;  lab:'GIR:'); (* GET_INTRPT     *)
   SAN: Tcmd = (code:$5C; qclass:cmd_nopara; lab:'SAN:'); (* SET_AUTO_UPDATE_ON*)
   SAF: Tcmd = (code:$5D; qclass:cmd_nopara; lab:'SAF:'); (* SET_AUTO_UPDATE_OFF*)
   TL : Tcmd = (code:$4C; qclass:rd_U1616;   lab:'TL:' ); (* GET_ACC        *)
//3
   ASN: Tcmd = (code:$45; qclass:cmd_nopara; lab:'ASN:'); (* SET_AUTO_STOP_ON *)
   ASF: Tcmd = (code:$44; qclass:cmd_nopara; lab:'ASF:'); (* SET_AUTO_STOP_OFF *)
   SMO: Tcmd = (code:$62; qclass:wr_nofrac;  lab:'SMO:'); (* SET_MTR_CMD    *)
   TE : Tcmd = (code:$E6; qclass:exdef;      lab:'TE:' ); (*                *)
   SFP: Tcmd = (code:$21; qclass:cmd_nopara; lab:'SFP:'); (* SET_FLTR_PID   *)
   SFV: Tcmd = (code:$22; qclass:cmd_nopara; lab:'SFV:'); (* SET_FLTR_PIVFF *)
   DP : Tcmd = (code:$25; qclass:wr_nofrac;  lab:'DP:' ); // SET_KP
   DD : Tcmd = (code:$27; qclass:wr_nofrac;  lab:'DD:' ); // SET_KD
   DI : Tcmd = (code:$26; qclass:wr_nofrac;  lab:'DI:' ); // SET_KI
   DF : Tcmd = (code:$2B; qclass:wr_nofrac;  lab:'DF:' ); // SET_KVFF
//4
   DL : Tcmd = (code:$28; qclass:wr_nofrac;  lab:'DL:' ); (* SET_I_LIM      *)
   GP : Tcmd = (code:$50; qclass:rd_nofrac;  lab:'GP:' ); (* GET_KP         *)
   GD : Tcmd = (code:$52; qclass:rd_nofrac;  lab:'GD:' ); (* GET_KD         *)
   GI : Tcmd = (code:$51; qclass:rd_nofrac;  lab:'GI:' ); (* GET_KI         *)
   GF : Tcmd = (code:$54; qclass:rd_nofrac;  lab:'GF:' ); (* GET_KVFF       *)
   GL : Tcmd = (code:$53; qclass:rd_nofrac;  lab:'GL:' ); (* GET_I_LIM      *)
   GPE: Tcmd = (code:$55; qclass:rd_nofrac;  lab:'GPE:'); (* GET_POS_ERR    *)
   GSI: Tcmd = (code:$2E; qclass:rd_nofrac;  lab:'GSI:'); (* GET_INTGR      *)
   RTI: Tcmd = (code:$32; qclass:wr_nofrac;  lab:'RTI:'); (* RST_INTRPT     *)
   GIM: Tcmd = (code:$56; qclass:rd_nofrac;  lab:'GIM:'); (* GET_INTRPT_MASK*)
//5
   TG : Tcmd = (code:$E7; qclass:exdef;      lab:'TG:' ); (* Tell Gain      *)
   SCI: Tcmd = (code:$64; qclass:cmd_nopara; lab:'SCI:'); (* SET_CAPT_INDEX *)
   LE : Tcmd = (code:$70; qclass:cmd_nopara; lab:'LE:' ); (* LMTS_ON        *)
   LD : Tcmd = (code:$71; qclass:cmd_nopara; lab:'LD:' ); (* LMTS_OFF       *)
   SPS: Tcmd = (code:$0B; qclass:cmd_nopara; lab:'SPS:'); (* SET_PRFL_S_CRV *)
   SPT: Tcmd = (code:$09; qclass:cmd_nopara; lab:'SPT:'); (* SET_PRFL_TRAP  *)
   SMA: Tcmd = (code:$15; qclass:wr_nofrac;  lab:'SMA:'); (* SET_MAX_ACC    *)
   GMA: Tcmd = (code:$4F; qclass:rd_nofrac;  lab:'GMA:'); (* GET_MAX_ACC    *)
   STP: Tcmd = (code:$46; qclass:exdef;      lab:'STP:'); (* CLEAR PROFILE  *)
   SYN: Tcmd = (code:$47; qclass:cmd_nopara; lab:'SYN:'); (* SYNCH_PRFL     *)
//6
   SPE: Tcmd = (code:$29; qclass:wr_nofrac;  lab:'SPE:'); (* SET_POS_ERR    *)
   SCH: Tcmd = (code:$65; qclass:cmd_nopara; lab:'SCH:'); (* SET_CAPT_HOME  *)
   SLS: Tcmd = (code:$66; qclass:wr_nofrac;  lab:'SLS:'); (* SET_LMT_SENSE  *)
   VE : Tcmd = (code:$6C; qclass:rd_nofrac;  lab:'VE:' ); (* GET_VRSN       *)
   GTI: Tcmd = (code:$3E; qclass:rd_nofrac;  lab:'GTI:'); (* GET_TIME       *)
   WA : Tcmd = (code:$E8; qclass:exdef;      lab:'WA:' ); (* Wait Absolute  *)
   SBT: Tcmd = (code:$17; qclass:cmd_nopara; lab:'SBT:'); (* SET_TIME_BRK   *)
   SBP: Tcmd = (code:$18; qclass:cmd_nopara; lab:'SBP:'); (* SET_POS_BRK    *)
   SBN: Tcmd = (code:$19; qclass:cmd_nopara; lab:'SBN:'); (* SET_NEG_BRK    *)
   SBA: Tcmd = (code:$1B; qclass:cmd_nopara; lab:'SBA:'); (* SET_ACTL_POS_BRK*)
//7
   SBC: Tcmd = (code:$1C; qclass:cmd_nopara; lab:'SBC:'); (* SET_ACTL_NEG_BRK*)
   SBF: Tcmd = (code:$6D; qclass:cmd_nopara; lab:'SBF:'); (* SET_BRK_OFF    *)
   SB : Tcmd = (code:$16; qclass:wr_nofrac;  lab:'SB:' ); (* SET_BRK_PNT    *)
   SIM: Tcmd = (code:$2F; qclass:wr_nofrac;  lab:'SIM:'); (* SET_INTRP_MASK *)
   GJ : Tcmd = (code:$58; qclass:rd_U0032;   lab:'GJ:' ); (* GET_JERK       *)
   SJ : Tcmd = (code:$13; qclass:wr_U0032;   lab:'SJ:' ); (* SET_JERK       *)
   SRA: Tcmd = (code:$14; qclass:wr_S1616;   lab:'SRA:'); (* SET_RATIO      *)
   GRA: Tcmd = (code:$59; qclass:rd_S1616;   lab:'GRA:'); (* GET_RATIO      *)
   SPV: Tcmd = (code:$0A; qclass:cmd_nopara; lab:'SPV:'); (* SET_PRFL_VEL   *)
   SPG: Tcmd = (code:$0C; qclass:cmd_nopara; lab:'SPG:'); (* SET_PRFL_GEAR  *)
//8
   SL : Tcmd = (code:$E9; qclass:exdef;      lab:'SL:' ); (* Set Limit Backstep*)
   AN : Tcmd = (code:$41; qclass:cmd_nopara; lab:'AN:' ); (* AXIS_ON        *)
   AF : Tcmd = (code:$40; qclass:cmd_nopara; lab:'AF:' ); (* AXIS_OFF       *)
   SOP: Tcmd = (code:$3C; qclass:cmd_nopara; lab:'SOP:'); (* SET_OUTPUT_PWM *)
   SOH: Tcmd = (code:$3B; qclass:cmd_nopara; lab:'SOH:'); (* SET_OUTPUT_DAC16*)
   SST: Tcmd = (code:$38; qclass:wr_nofrac;  lab:'SST:'); (* SET_SAMPLE_TIME*)
   GT : Tcmd = (code:$61; qclass:rd_nofrac;  lab:'GT:' ); (* GET_SMPL_TIME  *)
   GOM: Tcmd = (code:$6E; qclass:rd_nofrac;  lab:'GOM:'); (* GET_OUTPUT_MODE*)
   TZ : Tcmd = (code:$EA; qclass:exdef;      lab:'TZ:' ); (* Tell channel dez*)
   TC : Tcmd = (code:$EB; qclass:exdef;      lab:'TC:' ); (* Tell Channel    *)
//9
   CN : Tcmd = (code:$EC; qclass:exdef;      lab:'CN:' ); (* Channel ON    *)
   CF : Tcmd = (code:$ED; qclass:exdef;      lab:'CF:' ); (* Channel OFF   *)
   IV : Tcmd = (code:$EE; qclass:exdef;      lab:'IV:' );// Increment Velocity
   AB : Tcmd = (code:$EF; qclass:exdef;      lab:'AB:' ); (* Stop Abruptly *)
   WC : Tcmd = (code:$F0; qclass:exdef;      lab:'WC:' ); (* Wait for Channel*)
   WT : Tcmd = (code:$F1; qclass:exdef;      lab:'WT:' ); (* Wait stop trigger*)
   FEP: Tcmd = (code:$F2; qclass:exdef;      lab:'FEP:'); (* Find Edge Pos *)
   FEN: Tcmd = (code:$F3; qclass:exdef;      lab:'FEN:'); (* Find Edge Neg *)
   SAP: Tcmd = (code:$4D; qclass:wr_nofrac;  lab:'SAP:'); // SET_ACT_POS   
   BN : Tcmd = (code:$F4; qclass:exdef;      lab:'BN:' ); // Set brake ON
//10
   BF : Tcmd = (code:$F5; qclass:exdef;      lab:'BF:' ); // Set brake OFF
   RET: Tcmd = (code:$F6; qclass:exdef;      lab:'RET:' ); // Return from overcurrent
   SIL: Tcmd = (code:$F7; qclass:exdef;      lab:'SIL:' ); // cancel oscillations

//End of command assignment

(***************************************************************************)
(* Function Declarations  :
(***************************************************************************)
// Case sensitive writing required !!

function  get_pos(axis:byte):integer; stdcall external ExtLib; //2
function  get_PosErr(axis:byte):integer;  stdcall external ExtLib; //4
function  get_ErrorStatus:integer; stdcall external ExtLib; //5
function  get_stat(axis:byte):integer;stdcall external ExtLib; //6
function  get_RefSignal(axis:byte):word; stdcall external ExtLib; //7
procedure execute(axis:byte;command:TCmd;para:integer;report:Pchar);
                  stdcall external ExtLib; //10
procedure translate(cmd:Pchar;rep: Pchar); stdcall external ExtLib; //12
function  moving(axis:byte):boolean; stdcall external ExtLib; //14
procedure Set_BaseAddress(address:word); stdcall external ExtLib; //16
function  Axis_Installed:byte; stdcall external ExtLib; //18
function  InitBoard(mode:byte):integer; stdcall external ExtLib; //20
procedure InitAxis(axis:byte); stdcall external ExtLib; //22
procedure InitStage(axis,stage:byte); stdcall external ExtLib; //23
function  Board_Installed:boolean; stdcall external ExtLib; //24
procedure MoveA(axis:byte; position:integer); stdcall external ExtLib; //26
procedure MoveR(axis:byte; counts:integer); stdcall external ExtLib; // 28
function  WaitStop(axis:byte):integer; stdcall external ExtLib; //30
procedure Init_LS(drive1,drive2,drive3,drive4:byte); stdcall external ExtLib;//32
function  get_BoardStatus:word; stdcall external ExtLib; //34
procedure axis_select(axis:byte); stdcall external ExtLib; //36
procedure setQMC(cmd:byte; para:integer); stdcall external ExtLib; //38
function  getQMC(cmd:byte):integer; stdcall external ExtLib; //40
function  autodetect:integer; stdcall external ExtLib; //42
procedure set_pos(axis:byte; newpos:longint);stdcall external ExtLib; //44
function  get_v(axis:byte):integer; stdcall external ExtLib; //46

procedure AutoFindEdge(axis:byte; mode:byte); stdcall external ExtLib; //70
function  get_DIN:byte; stdcall external ExtLib; 
procedure set_DOUT(digout:byte); stdcall external ExtLib; 
function  set_Amplifier(mode:byte):integer; stdcall external ExtLib; 
function  get_Limit:integer; stdcall external ExtLib; 
function  get_LimitAxis(axis:byte):integer; stdcall external ExtLib; 
procedure set_pid(axis:byte; pterm,iterm,dterm:integer); stdcall external ExtLib; 
procedure set_vel(axis:byte; velocity:integer); stdcall external ExtLib; 
procedure set_acc(axis:byte; acceleration:integer); stdcall external ExtLib;
procedure VectorA(axis_1, axis_2:byte; pos_1, pos_2, vv,va:integer);
          stdcall external ExtLib; //60
procedure VectorR(axis_1,axis_2:byte; move_1,move_2,vv,va:integer);
          stdcall external ExtLib; //61
procedure MoveA_12(p1,p2:integer); stdcall external ExtLib; // 50
procedure MoveA_123(p1,p2,p3:integer); stdcall external ExtLib; // 51
procedure MoveR_12(d1,d2:integer); stdcall external ExtLib; // 52
procedure MoveR_123(d1,d2,d3:integer); stdcall external ExtLib; // 53


//------------------------------------------------------------------------------
// End of file


